-- ************************************************************************
-- **        ██████  ██████   █████  ██    ██ ███████ ███    ██          **
-- **        ██   ██ ██   ██ ██   ██ ██    ██ ██      ████   ██          **
-- **        ██████  ██████  ███████ ██    ██ █████   ██ ██  ██          **
-- **        ██   ██ ██   ██ ██   ██  ██  ██  ██      ██  ██ ██          **
-- **        ██████  ██   ██ ██   ██   ████   ███████ ██   ████          **
-- ************************************************************************
-- ** All rights reserved. This content is protected by © Copyright law. **
-- ************************************************************************

local requiredMaterials = {}

local function onGameStart()
	if SandboxVars.AutoEngineering.NeedsRepairMaterials then
		requiredMaterials.blowtorchFuel = SandboxVars.AutoEngineering.RepairFuel
		requiredMaterials.EngineParts = SandboxVars.AutoEngineering.RepairEngineParts
		requiredMaterials.SheetMetal = SandboxVars.AutoEngineering.RepairSheetMetal
		requiredMaterials.ScrapMetal = SandboxVars.AutoEngineering.RepairScrapMetal
		requiredMaterials.ElectronicsScrap = SandboxVars.AutoEngineering.RepairElectronicsScrap
	else
		requiredMaterials.blowtorchFuel = 0
		requiredMaterials.EngineParts = 0
		requiredMaterials.SheetMetal = 0
		requiredMaterials.ScrapMetal = 0
		requiredMaterials.ElectronicsScrap = 0
	end
end

Events.OnGameStart.Add(onGameStart)

local function tryFixWreck(playerObj, vehicle)
	ISTimedActionQueue.add(ISPathFindAction:pathToVehicleAdjacent(playerObj, vehicle))
	ISTimedActionQueue.add(ISAETimeAction:FixWreck(playerObj, vehicle, 5000))
end

local function tryInstallEngine(playerObj, engineItem, vehicle)
	ISTimedActionQueue.add(ISPathFindAction:pathToVehicleAdjacent(playerObj, vehicle))
	ISTimedActionQueue.add(ISAETimeAction:InstallEngine(playerObj, engineItem, vehicle, 1000))
end

local function tryRemoveEngine(playerObj, vehicle)
	ISTimedActionQueue.add(ISPathFindAction:pathToVehicleAdjacent(playerObj, vehicle))
	ISTimedActionQueue.add(ISAETimeAction:RemoveEngine(playerObj, vehicle, 1000))
end

local function predicateWeldingMask(item)
	return item:hasTag("WeldingMask") or item:getType() == "WeldingMask"
end

local function getRequirements(player, vehicleMechanicType, inventory)

	local playerObj = getSpecificPlayer(player)
	local args = {}

	if not playerObj:isRecipeKnown("RepairWreckType" .. vehicleMechanicType .. "") and SandboxVars.AutoEngineering.NeedsWreckRecipe then
		args.knowsRecipe = false
	else
		args.knowsRecipe = true
	end

	args.Mechanics = playerObj:getPerkLevel(Perks.Mechanics)
	args.MetalWelding = playerObj:getPerkLevel(Perks.MetalWelding)
	args.Electricity = playerObj:getPerkLevel(Perks.Electricity)

	args.EnginePartsCount = ISBuildMenu.countMaterial(player, "Base.EngineParts")
	args.SheetMetalCount = ISBuildMenu.countMaterial(player, "Base.SheetMetal")
	args.ScrapMetalCount = ISBuildMenu.countMaterial(player, "Base.ScrapMetal")
	args.ElectronicsScrapCount = ISBuildMenu.countMaterial(player, "Base.ElectronicsScrap")

	if inventory:containsEvalRecurse(predicateWeldingMask) then
		args.hasWeldingMask = true
	else
		args.hasWeldingMask = false
	end

	local blowTorch = ISBlacksmithMenu.getBlowTorchWithMostUses(inventory)

	if blowTorch then
		args.hasBlowtorch = true
		args.blowtorchFuel = blowTorch:getDrainableUsesInt()
	else
		args.hasBlowtorch = false
	end

	return args
end

local OnShowRadialMenuOutside = ISVehicleMenu.showRadialMenuOutside

ISVehicleMenu.showRadialMenuOutside = function(playerObj)

	if playerObj:getVehicle() then return end

	OnShowRadialMenuOutside(playerObj)

	local vehicle = playerObj:getNearVehicle(); if not vehicle then return end
	if vehicle:isEngineStarted()  then return end
	if vehicle:isEngineRunning()  then return end

	local playerIndex = playerObj:getPlayerNum()
	local menu = getPlayerRadialMenu(playerIndex)

	local isSpecialUser = isDebugEnabled()
	if not isSpecialUser then isSpecialUser = isAdmin() end

	if playerObj:getPerkLevel(Perks.Mechanics) >= SandboxVars.AutoEngineering.EngineMechanicsLvl or isSpecialUser then

		local engine = vehicle:getPartById("Engine")
		if (engine and engine:getCategory() ~= "nodisplay") then
			if engine:getCondition() >= 5 then
				menu:addSlice(getText("ContextMenu_Remove_Engine"), getTexture("profession_engineer"), tryRemoveEngine, playerObj, vehicle)
			end
		end
	end

	if BravensWreckFixMain.isWreck(vehicle) then
		if not isSpecialUser then

			local inventory = playerObj:getInventory()
			local vehicleMechanicType = vehicle:getScript():getMechanicType()

			local requirements = getRequirements(playerIndex, vehicleMechanicType, inventory)
			if requirements.knowsRecipe == false then return end
			if requirements.Mechanics < SandboxVars.AutoEngineering.RepairMechanicsLvl then return end
			if requirements.MetalWelding < SandboxVars.AutoEngineering.RepairWeldingLvl then return end
			if requirements.Electricity < SandboxVars.AutoEngineering.RepairElectricityLvl then return end
			if requirements.hasWeldingMask == false then return end
			if requirements.hasBlowtorch == false then return end
			if requirements.blowtorchFuel < requiredMaterials.blowtorchFuel then return end
			if requirements.EnginePartsCount < requiredMaterials.EngineParts then return end
			if requirements.SheetMetalCount < requiredMaterials.SheetMetal then return end
			if requirements.ScrapMetalCount < requiredMaterials.ScrapMetal then return end
			if requirements.ElectronicsScrapCount < requiredMaterials.ElectronicsScrap then return end
		end

		menu:addSlice(getText("ContextMenu_Repair_Wreck"), getTexture("profession_engineer"), tryFixWreck, playerObj, vehicle)
	end
end

local OnFillMenuOutsideVehicle = ISVehicleMenu.FillMenuOutsideVehicle

ISVehicleMenu.FillMenuOutsideVehicle = function(player, context, vehicle, test)

	if vehicle:isEngineStarted()  then return end
	if vehicle:isEngineRunning()  then return end

	local playerObj = getSpecificPlayer(player)
	local inventory = playerObj:getInventory()

	local isSpecialUser = isDebugEnabled()
	if not isSpecialUser then isSpecialUser = isAdmin() end

	if playerObj:getPerkLevel(Perks.Mechanics) >= SandboxVars.AutoEngineering.EngineMechanicsLvl or isSpecialUser then

		local engine = vehicle:getPartById("Engine")
		if (engine and engine:getCategory() ~= "nodisplay") then

			local removeOption = context:addOptionOnTop(getText("ContextMenu_Remove_Engine"), playerObj, tryRemoveEngine, vehicle)
			local tooltip = ISWorldObjectContextMenu.addToolTip()
			tooltip:setName(getText("ContextMenu_Remove_Engine"))

			removeOption.toolTip = tooltip
			removeOption.iconTexture = getTexture("profession_engineer")

			if engine:getCondition() >= 5 then
				tooltip.description = "<LINE> <RGB:1,1,1>" .. getText("Tooltip_RemoveEngineInfo") .. ""
			else
				tooltip.description = "<LINE> <RGB:1,0,0>" .. getText("Tooltip_EngineUnusable") .. ""
				removeOption.notAvailable = true
			end

			local engineArray = playerObj:getInventory():getItemsFromType("AEEngine")
			if (engine:getCondition() == 0 and engineArray:size() ~= 0) then

				local installOption = context:addOptionOnTop(getText("ContextMenu_Install_Engine"))
				installOption.iconTexture = getTexture("profession_engineer")

				local insertEngineContext = ISContextMenu:getNew(context)
				context:addSubMenu(installOption, insertEngineContext)

				for i = 0, engineArray:size()-1 do
					local engineItem = engineArray:get(i)
					insertEngineContext:addOption(engineItem:getName() .. " (" .. (engineItem:getModData().horsePower / 10) .. " HP" .. ") " , playerObj, tryInstallEngine, engineItem, vehicle)
				end
			end
		end
	else
		local removeOption = context:addOptionOnTop(getText("ContextMenu_Remove_Engine"), playerObj, tryRemoveEngine, vehicle)
		removeOption.notAvailable = true

		local tooltip = ISWorldObjectContextMenu.addToolTip()
		tooltip.description = "<LINE> <RGB:1,0,0>" .. getText("Tooltip_CantRemoveEngineInfo"):gsub("%[LEVEL%]", SandboxVars.AutoEngineering.EngineMechanicsLvl)
		tooltip:setName(getText("ContextMenu_Remove_Engine"))

		removeOption.toolTip = tooltip
		removeOption.iconTexture = getTexture("profession_engineer")
	end

	if BravensWreckFixMain.isWreck(vehicle) then

		local vehicleMechanicType = vehicle:getScript():getMechanicType()
		local repairOption = context:addOptionOnTop(getText("ContextMenu_Repair_Wreck"), playerObj, tryFixWreck, vehicle)
		local tooltip = ISWorldObjectContextMenu.addToolTip()
		repairOption.iconTexture = getTexture("profession_engineer")

		if not isSpecialUser then

			local requirements = getRequirements(player, vehicleMechanicType, inventory)

			if requirements.knowsRecipe == false then
				tooltip:setName(getText("ContextMenu_Repair_Wreck"))
				tooltip.description = "<LINE> <RGB:1,0,0>" .. getText("Tooltip_NeedsWreckMagazine") .. getItemNameFromFullType("Braven.AutoEngineerMagVol" .. vehicleMechanicType .. "") .. ""
				repairOption.toolTip = tooltip
				repairOption.notAvailable = true
			else
				tooltip:setName(getText("ContextMenu_Repair_Wreck"))
				tooltip.description = getText("Tooltip_WreckFixInfo") .. " <LINE> <LINE> "


				if requirements.Mechanics >= SandboxVars.AutoEngineering.RepairMechanicsLvl then
					tooltip.description = tooltip.description .. " <LINE> <RGB:1,1,1> " ..  getText("IGUI_perks_Mechanics") .. " " .. " " .. requirements.Mechanics .. "/" .. 5
				else
					tooltip.description = tooltip.description .. " <LINE> <RGB:1,0,0> " .. getText("IGUI_perks_Mechanics") .. " " .. " " .. requirements.Mechanics .. "/" .. 5
					repairOption.notAvailable = true
				end

				if requirements.MetalWelding >= SandboxVars.AutoEngineering.RepairWeldingLvl then
					tooltip.description = tooltip.description .. " <LINE> <RGB:1,1,1> " ..  getText("IGUI_perks_Mechanics") .. " " .. " " .. requirements.Mechanics .. "/" .. 5
				else
					tooltip.description = tooltip.description .. " <LINE> <RGB:1,0,0> " .. getText("IGUI_perks_Mechanics") .. " " .. " " .. requirements.Mechanics .. "/" .. 5
					repairOption.notAvailable = true
				end

				if requirements.Electricity >= SandboxVars.AutoEngineering.RepairElectricityLvl then
					tooltip.description = tooltip.description .. " <LINE> <RGB:1,1,1> " ..  getText("IGUI_perks_Mechanics") .. " " .. " " .. requirements.Mechanics .. "/" .. 5
				else
					tooltip.description = tooltip.description .. " <LINE> <RGB:1,0,0> " .. getText("IGUI_perks_Mechanics") .. " " .. " " .. requirements.Mechanics .. "/" .. 5
					repairOption.notAvailable = true
				end

				if not repairOption.notAvailable then

					if requirements.EnginePartsCount >= requiredMaterials.EngineParts then
						tooltip.description = tooltip.description .. " <LINE> <RGB:1,1,1> " .. getItemNameFromFullType("Base.EngineParts") .. " " .. " " .. requirements.EnginePartsCount .. "/" .. requiredMaterials.EngineParts
					else
						tooltip.description = tooltip.description .. " <LINE> <RGB:1,0,0> " .. getItemNameFromFullType("Base.EngineParts") .. " " .. " " .. requirements.EnginePartsCount .. "/" .. requiredMaterials.EngineParts
						repairOption.notAvailable = true
					end

					if requirements.SheetMetalCount >= requiredMaterials.SheetMetal then
						tooltip.description = tooltip.description .. " <LINE> <RGB:1,1,1> " .. getItemNameFromFullType("Base.SheetMetal") .. " " .. " " .. requirements.SheetMetalCount .. "/" .. requiredMaterials.SheetMetal
					else
						tooltip.description = tooltip.description .. " <LINE> <RGB:1,0,0> " .. getItemNameFromFullType("Base.SheetMetal") .. " " .. " " .. requirements.SheetMetalCount .. "/" .. requiredMaterials.SheetMetal
						repairOption.notAvailable = true
					end

					if requirements.ScrapMetalCount >= requiredMaterials.ScrapMetal then
						tooltip.description = tooltip.description .. " <LINE> <RGB:1,1,1> " .. getItemNameFromFullType("Base.ScrapMetal") .. " " .. " " .. requirements.ScrapMetalCount .. "/" .. requiredMaterials.ScrapMetal
					else
						tooltip.description = tooltip.description .. " <LINE> <RGB:1,0,0> " .. getItemNameFromFullType("Base.ScrapMetal") .. " " .. " " .. requirements.ScrapMetalCount .. "/" .. requiredMaterials.ScrapMetal
						repairOption.notAvailable = true
					end

					if requirements.ElectronicsScrapCount >= requiredMaterials.ElectronicsScrap then
						tooltip.description = tooltip.description .. " <LINE> <RGB:1,1,1> " .. getItemNameFromFullType("Base.ElectronicsScrap") .. " " .. " " .. requirements.ElectronicsScrapCount .. "/" .. requiredMaterials.ElectronicsScrap
					else
						tooltip.description = tooltip.description .. " <LINE> <RGB:1,0,0> " .. getItemNameFromFullType("Base.ElectronicsScrap") .. " " .. " " .. requirements.ElectronicsScrapCount .. "/" .. requiredMaterials.ElectronicsScrap
						repairOption.notAvailable = true
					end

					if requirements.hasWeldingMask == true then
						tooltip.description = tooltip.description .. " <LINE> <RGB:1,1,1> " .. getItemNameFromFullType("Base.WeldingMask") .. " 1/1"
					else
						tooltip.description = tooltip.description .. " <LINE> <RGB:1,0,0> " .. getItemNameFromFullType("Base.WeldingMask") .. " 0/1"
						repairOption.notAvailable = true
					end

					if requirements.hasBlowtorch == true then

						if requirements.blowtorchFuel >= requiredMaterials.blowtorchFuel then
							tooltip.description = tooltip.description .. " <LINE> <RGB:1,1,1> " .. getItemNameFromFullType("Base.BlowTorch") .. " " .. getText("ContextMenu_Uses") .. " " .. requirements.blowtorchFuel .. "/" .. requiredMaterials.blowtorchFuel
						else
							tooltip.description = tooltip.description .. " <LINE> <RGB:1,0,0> " .. getItemNameFromFullType("Base.BlowTorch") .. " " .. getText("ContextMenu_Uses") .. " " .. requirements.blowtorchFuel .. "/" .. requiredMaterials.blowtorchFuel
							repairOption.notAvailable = true
						end
					else
						tooltip.description = tooltip.description .. " <LINE> <RGB:1,0,0> " .. getItemNameFromFullType("Base.BlowTorch") .. " 0/1"
						repairOption.notAvailable = true
					end
				end
			end
		else
			tooltip:setName(getText("ContextMenu_Repair_Wreck"))
			tooltip.description = getText("Tooltip_WreckFixInfo") .. " <LINE> <LINE> "
			tooltip.description = tooltip.description .. " <LINE> <RGB:1,1,1> " .. getItemNameFromFullType("Base.EngineParts") .. " " .. " " .. requiredMaterials.EngineParts .. "/" .. requiredMaterials.EngineParts
			tooltip.description = tooltip.description .. " <LINE> <RGB:1,1,1> " .. getItemNameFromFullType("Base.SheetMetal") .. " " .. " " .. requiredMaterials.SheetMetal .. "/" .. requiredMaterials.SheetMetal
			tooltip.description = tooltip.description .. " <LINE> <RGB:1,1,1> " .. getItemNameFromFullType("Base.ScrapMetal") .. " " .. " " .. requiredMaterials.ScrapMetal .. "/" .. requiredMaterials.ScrapMetal
			tooltip.description = tooltip.description .. " <LINE> <RGB:1,1,1> " .. getItemNameFromFullType("Base.ElectronicsScrap") .. " " .. " " .. requiredMaterials.ElectronicsScrap .. "/" .. requiredMaterials.ElectronicsScrap
			tooltip.description = tooltip.description .. " <LINE> <RGB:1,1,1> " .. getItemNameFromFullType("Base.BlowTorch") .. " " .. getText("ContextMenu_Uses") .. " " .. requiredMaterials.blowtorchFuel .. "/" .. requiredMaterials.blowtorchFuel
		end

		repairOption.toolTip = tooltip
	end

	OnFillMenuOutsideVehicle(player, context, vehicle, test)
end

BravensWreckFixMain = {}

BravensWreckFixMain.isWreck = function(vehicle)

	local vehicleIsWreck = false
	local vehicleName

	if not BravensUtils.isNumber(vehicle) then
		vehicleName = vehicle:getScriptName()
	else
		vehicleName = getVehicleById(vehicle):getScriptName()
	end

	if not vehicleName then return vehicleIsWreck end

	if string.contains(vehicleName, "Smashed") then
		vehicleIsWreck = true
	end

	return vehicleIsWreck
end

BravensWreckFixMain.FixWreck = function(playerObj, vehicle)

	local inventory = playerObj:getInventory()

	if not(isDebugEnabled() or isAdmin()) then

		for _ = 1, requiredMaterials.EngineParts, 1 do
			local item = inventory:getItemFromType("Base.EngineParts")
			if item then inventory:Remove(item) end
		end

		for _ = 1, requiredMaterials.SheetMetal, 1 do
			local item = inventory:getItemFromType("Base.SheetMetal")
			if item then inventory:Remove(item) end
		end

		for _ = 1, requiredMaterials.ScrapMetal, 1 do
			local item = inventory:getItemFromType("Base.ScrapMetal")
			if item then inventory:Remove(item) end
		end

		for _ = 1, requiredMaterials.ElectronicsScrap, 1 do
			local item = inventory:getItemFromType("Base.ElectronicsScrap")
			if item then inventory:Remove(item) end
		end

		local blowTorch = ISBlacksmithMenu.getBlowTorchWithMostUses(inventory)
		if blowTorch then
			local fuelLeft = blowTorch:getDrainableUsesInt()
			local fuelRequired = requiredMaterials.blowtorchFuel
			if fuelLeft >= fuelRequired then
				blowTorch:setUses(fuelLeft - fuelRequired)
			end
		end
	end

	AEClient.SpawnVehicle(vehicle)
	BravensUtils.TirePlayer(playerObj, 0.3)
end